<?php
/*
Plugin Name:  Password Protect Website
Plugin URI:   https://www.limaweb.nl/downloads/
Description:  Restrict access to WordPress website with a password
Version:      1.0
Author:       LimaWeb
Author URI:   https://www.limaweb.nl/
License:      GPL2
License URI:  https://www.gnu.org/licenses/gpl-2.0.html
*/


if (! defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

// Admin Settings Page
function pp_register_settings()
{
    register_setting('pp_settings_group', 'pp_site_password', 'pp_hash_password');
    register_setting('pp_settings_group', 'pp_site_password_image');
    add_settings_section(
        'pp_settings_section',
        'Password Protection Settings',
        '__return_null',
        'password-protect-site'
    );
    add_settings_field(
        'pp_site_password',
        'Site Password',
        'pp_password_field_callback',
        'password-protect-site',
        'pp_settings_section'
    );
    add_settings_field(
        'pp_site_password_image',
        'Password Form Image',
        'pp_image_field_callback',
        'password-protect-site',
        'pp_settings_section'
    );
}
add_action('admin_init', 'pp_register_settings');

function pp_password_field_callback()
{
    echo '<input type="password" name="pp_site_password" class="regular-text" />';
}

function pp_image_field_callback()
{
    $image_url = get_option('pp_site_password_image', '');
?>
    <img id="pp-image-preview" src="<?php echo esc_url($image_url); ?>" style="<?php echo $image_url ? '' : 'display:none'; ?>; max-width: 300px; max-height: 300px;"><br>
    <input type="text" id="pp-image-url" name="pp_site_password_image" value="<?php echo esc_url($image_url); ?>" class="regular-text" style="display:none;">
    <input type="button" class="button-primary" value="<?php _e('Select Image', 'textdomain'); ?>" id="pp-select-image">
    <input type="button" class="button-secondary" value="<?php _e('Remove Image', 'textdomain'); ?>" id="pp-remove-image" style="<?php echo $image_url ? '' : 'display:none'; ?>;">
<?php
}

function pp_settings_page()
{
?>
    <div class="wrap">
        <h1>Password Protect Entire Site</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('pp_settings_group');
            do_settings_sections('password-protect-site');
            submit_button();
            ?>
        </form>
    </div>
<?php
}

function pp_add_settings_page()
{
    add_options_page(
        'Password Protect Site',
        'Password Protect Site',
        'manage_options',
        'password-protect-site',
        'pp_settings_page'
    );
}
add_action('admin_menu', 'pp_add_settings_page');

function pp_hash_password($password)
{
    if (empty($password)) {
        return get_option('pp_site_password');
    }
    return password_hash($password, PASSWORD_DEFAULT);
}

// Load WP media scripts
function pp_enqueue_admin_scripts()
{
    wp_enqueue_media();
    wp_enqueue_script('pp-admin-script', plugin_dir_url(__FILE__) . 'pp-admin.js', array('jquery'), null, true);
}
add_action('admin_enqueue_scripts', 'pp_enqueue_admin_scripts');

// Display Password Form
function pp_display_password_form($message = '')
{
    $image_url = get_option('pp_site_password_image', '');
?>
    <style>
        .pp-password-form {
            text-align: center;
            margin-top: 50px;
        }

        .pp-password-form img {
            margin-bottom: 20px;
        }

        .pp-password-form label {
            display: block;
            font-weight: bold;
        }

        .pp-password-form input[type="password"] {
            margin-top: 10px;
            margin-bottom: 20px;
            display: block;
            width: 100%;
            max-width: 300px;
            margin-left: auto;
            margin-right: auto;
        }
    </style>
    <form class="pp-password-form" action="" method="post">
        <?php if ($image_url): ?>
            <img src="<?php echo esc_url($image_url); ?>" alt="Password Form Image" style="max-width: 300px; max-height: 300px;"><br>
        <?php endif; ?>
        <p>
            <label for="site-password">Enter Password to Access the Site:</label>
            <input type="password" id="site-password" name="site_password">
            <input type="submit" value="Submit">
        </p>
        <?php if ($message): ?>
            <p style="color: red;"><?php echo $message; ?></p>
        <?php endif; ?>
    </form>
<?php
}

function pp_check_password()
{
    if (session_status() === PHP_SESSION_NONE) {
        session_start();
    }

    // Check if the form is submitted
    if (isset($_POST['site_password'])) {
        $submitted_password = sanitize_text_field($_POST['site_password']);
        $stored_password    = get_option('pp_site_password');
        $ip_address         = $_SERVER['REMOTE_ADDR'];

        // Get user-agent
        $user_agent = isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '';

        // Block users with empty user-agent or known bot user-agents
        if (empty($user_agent) || pp_is_bot_user_agent($user_agent)) {
            wp_die(__('Access denied.', 'textdomain'), 'Access Restricted');
        }

        // Get the current attempts from transient
        $attempts = get_transient('pp_login_attempts_' . $ip_address) ?: [];

        // Filter to remove attempts older than 5 minutes
        $attempts = array_filter($attempts, function ($timestamp) {
            return $timestamp > time() - 300;
        });

        // Check if the user is blocked
        if (count($attempts) >= 5) {
            // Check if the user should be blocked for 15 minutes
            $attempts[] = time();
            if (count($attempts) > 10) {
                set_transient('pp_login_attempts_block_' . $ip_address, true, 15 * MINUTE_IN_SECONDS);
                $attempts = [];
            }
            set_transient('pp_login_attempts_' . $ip_address, $attempts, 5 * MINUTE_IN_SECONDS);
            wp_die(pp_display_password_form('Too many login attempts. Please try again in 15 minutes.'), 'Access Restricted');
        }

        // Validate password
        if (password_verify($submitted_password, $stored_password)) {
            $cookie_value = bin2hex(random_bytes(32));
            setcookie('pp_site_access', $cookie_value, time() + 3600, COOKIEPATH, COOKIE_DOMAIN, true, true);
            $_SESSION['pp_site_access'] = $cookie_value;

            wp_redirect(home_url($_SERVER['REQUEST_URI']));
            exit;
        } else {
            // Add current timestamp to attempts
            $attempts[] = time();
            set_transient('pp_login_attempts_' . $ip_address, $attempts, 5 * MINUTE_IN_SECONDS);
        }
    }
}
add_action('init', 'pp_check_password');

// Function to check known bot user-agents
function pp_is_bot_user_agent($user_agent)
{
    $bot_user_agents = [
        'python',
        'curl',
        'libwww-perl',
        'wget',
        'java',
        'php',
        'node',
        'go-http-client',
        'ruby',
        'http_client'
    ];

    foreach ($bot_user_agents as $bot_user_agent) {
        if (stripos($user_agent, $bot_user_agent) !== false) {
            return true;
        }
    }
    return false;
}

// Restrict Access to the Site
function pp_restrict_access()
{
    if (session_status() === PHP_SESSION_NONE) {
        session_start();
    }

    if (is_admin() || strpos($_SERVER['REQUEST_URI'], 'wp-login.php') !== false) {
        return;
    }

    $ip_address = $_SERVER['REMOTE_ADDR'];
    $is_blocked = get_transient('pp_login_attempts_block_' . $ip_address);

    // Get user-agent
    $user_agent = isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '';

    // Block users with empty user-agent or known bot user-agents
    if (empty($user_agent) || pp_is_bot_user_agent($user_agent)) {
        wp_die(__('Access denied.', 'textdomain'), 'Access Restricted');
    }

    if ($is_blocked) {
        wp_die(pp_display_password_form('Too many login attempts. Please try again in 15 minutes.'), 'Access Restricted');
    }

    if (! isset($_COOKIE['pp_site_access']) || ! isset($_SESSION['pp_site_access']) || $_COOKIE['pp_site_access'] !== $_SESSION['pp_site_access']) {
        wp_die(pp_display_password_form(), 'Access Restricted');
    }
}
add_action('template_redirect', 'pp_restrict_access');
?>